/*
** Copyright (c) 2012 Pierre-Jean Fichet.
** All rights reserved.
** 
** Redistribution and use in source and binary forms, with or
** without modification, are permitted provided that the
** following conditions are met:
**  1. Redistributions of source code must retain the above
** copyright notice, this list of conditions and the following
** disclaimer.
**  2. Redistributions in binary form must reproduce the above
** copyright notice, this list of conditions and the following
** disclaimer in the documentation and/or other materials
** provided with the distribution.
** 
** THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS
** ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
** BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
** EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
** HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
** OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
** $Id: dpage.c,v 0.2 2013/07/14 10:26:25 pj Exp $
*/


/*
** dpage.c
** Put two A4 pages on an A3 sheet of paper
*/




#define GETMEDIA "%%DocumentMedia: x595y842 595 842 0 () ()"
#define PUTMEDIA "%%DocumentMedia: x1190y842 1190 842 0 () ()"
#define GETORIENT "/landscape true def"
#define PUTORIENT "/landscape true def"
#define GETBBOX "/pagebbox [0 0 595 842] def"
#define PUTBBOX "/pagebbox [0 0 1190 842] def"
#define GETDEVICE "/setpagedevice where {pop 1 dict dup /PageSize [595 842] put setpagedevice} if"
#define PUTDEVICE "/setpagedevice where {pop 1 dict dup /PageSize [1190 842] put setpagedevice} if"
#define GETBLANK "%%Page: * "
#define GETPAGE "%%Page: "
#define GETENDPAGE "%%EndPage: "
#define GETSHOWPAGE "showpage"
#define GETSAVE "save"
#define PUTTRANSLATE "590000 0 translate"


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*
** psbook add pages in the form:
** %%Page: * n
** showpage
** we want:
** %%Page: * n
** save
** showpage
** restore
** %%EndPage: * n
*/



int main(int argc, char **argv)
{
	FILE *in; // input file
	char *line=0;
	size_t linesize = 128;
	char c;
	int n=0; // character per line
	int p=0; // Page number
	int o=1; // Output page
	int b=0; // blank page (added by pspage)

	argc--;
	argv++;
	if (argc==0)
		in=stdin;
	else {
		in = fopen(argv[0], "r");
		if (in == 0)
			perror(argv[0]);
	}

	line=malloc(linesize);
	if (line == 0)
		exit(1);
		
	while ( (c=getc(in))!=EOF) {
		if (n+1 >= linesize)
			line=realloc(line, linesize += 128);

		if (c== '\n'){
			line[n]='\0';

			if (!strcmp(line, GETMEDIA))
				printf("%s\n", PUTMEDIA);
			else if (!strcmp(line, GETORIENT))
				printf("%s\n", PUTORIENT);
			else if (!strcmp(line, GETBBOX))
				printf("%s\n", PUTBBOX);
			else if (!strcmp(line, GETDEVICE))
				printf("%s\n", PUTDEVICE);
			else if (!strncmp(line, GETPAGE, 7)){
				printf("%% %s\n", line);
				if (o == 1) {
					p++;
					printf("%s%d %d\n", GETPAGE, p, p);
					o=0;
				}
				else
					o=1;
				// take care of blank page added by psbook
				if (!strncmp(line, GETBLANK, 9)){
					b=1;
					printf("save\n");
					if (o==1)
						printf("%s\n", PUTTRANSLATE);
				}
			}
			else if (!strncmp(line, GETENDPAGE, 10)){
				printf("%% %s\n", line);
				if (o == 1)
					printf("%s%d %d\n", GETENDPAGE, p, p);
			}
			else if (!strcmp(line, GETSHOWPAGE)){
				if (o==0)
					printf("%% %s\n", line);
				else
					printf("%s\n", line);
				// take care of blank page added by psbook
				if (b == 1) {
					printf("restore\n");
					printf("%% %s*\n", GETENDPAGE);
					if (o == 1)
					printf("%s%d %d\n", GETENDPAGE, p, p);
					b=0;
				}
			}
			else if (!strcmp(line, GETSAVE) && o==1){
				printf("%s\n", line);
				printf("%s\n", PUTTRANSLATE);
			}
			else 
				printf("%s\n", line);
			n=0;
		} // end newline
		else
			line[n++] = c;
	} // end while loop

	line[n]='\0';
	printf("%s\n", line);

	return 0;
}

	
